@@ -3,8 +3,9 @@
 ңУǺץ
 	 ꥨ ͥåȥңУǥ <ͥåȥط饤֥>
 	 Version 0.001, October 2000
+    $Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $
 	 ------------------------------------------------------------
-	 Copyright (C) 2000,2001,2002ǯ naoki kuwamori<naoki@kuwamori.com>
+	 Copyright (C) 2000,2001,2002,2003ǯ naoki kuwamori<naoki@kuwamori.com>
 
 	 ܥץϥե꡼եȥǤʤϡFree Software Foundation
 	 ɽGNU ̸ͭѵΡ֥С󣲡װϤʹߤγƥС
@@ -17,7 +18,8 @@
 
 	Laurier Net work RPG server<game network library>
 	Version 0.001, October 2000
-	Copyright (C) 2000,2001,2002 Naoki Kuwamori <naoki@kuwamori.com>
+    $Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $
+	Copyright (C) 2000,2001,2002,2003 Naoki Kuwamori <naoki@kuwamori.com>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -29,6 +31,7 @@
 
 /etc/servicese game_rpgrpg_chatrpg_clientΥȥɬפ롣
 */
+static char cvs_id[] = "$Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $";
 
 
 /*
@@ -64,7 +67,7 @@ static int servfdUD = 0;
 static int rpg_fdnom = 0;
 struct rlimit fdlim;
 static int flag = 0;
-
+#define  RPGDEBUGLEVEL 1000
 
 void brakepipe(int data){
 	rpglog (RPGDEBUGLEVEL ,"libnet:get broken pipe\n");
@@ -76,7 +79,6 @@ void brakepipe(int data){
 }	 
 
 void  rpg_childdeth(int data){
-struct rusage rusage;
 int status;
 	rpglog(RPGDEBUGLEVEL , "libnet:Is child prosess die?\n");
 	while (waitpid (-1,&status,WNOHANG)>0){
@@ -100,7 +102,7 @@ void rpg_endproc(int data){
 	/*ƤΣƣĤĤ*/
 	for (cnt = 0 ; cnt < rpg_fdnom ; cnt ++){
 		if (rpg_FDS[cnt] == 0)continue;
-		rpgclosefd (rpg_FDS[cnt]);
+		rpg_closefd (rpg_FDS[cnt]);
 	}
 	if (rpg_FDS != NULL){free (rpg_FDS);rpg_FDS= NULL;}
 	if (s_Poll != NULL){free (s_Poll);s_Poll= NULL;}
@@ -115,7 +117,7 @@ void rpg_endproc(int data){
 /*
 	ƣĤơ֥¸ߤ뤫ǧ롣
 */
-int test_fd(int fd ){
+static int test_fd(int fd ){
 int cnt;
 
 	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
@@ -127,7 +129,7 @@ int cnt;
 }
 
 /*ƣĤΥǡ١餢Ƥõ֤*/
-int getfdnewentry(){
+static int getfdnewentry(){
 int cnt;
 	/*ޤƤʤä*/
 	clear_fddatabase();
@@ -157,7 +159,7 @@ int cnt;
 }
 
 /*ΣƣĤ򥯥ơǡ١*/
-void	rpgclosefd(int FD){
+void	rpg_closefd(int FD){
 int cnt;
 
 	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
@@ -226,7 +228,7 @@ int cnt;
 ǽΰ󤷤򤷤ʤΤǤޤäˤˤ
 rpg_fd_allcloseȤɬפ*/
 
-int clear_fddatabase(){
+static int clear_fddatabase(){
 int cnt;
 	
 	/*ǽλϣƣĤޤƽ򤹤*/
@@ -318,6 +320,22 @@ int cnt;
 	return LIBNETERR;
 }
 
+/*ƣĥǡ١˥ݡ󥰤볰ƣĤϿ*/
+int rpg_addRawPollFD(int FD){
+int ent;
+	if (test_fd(FD)!=LIBNETNOFD){
+		rpglog ( RPGDEBUGLEVEL , "libnet:The FD aredy seted\n");
+		return LIBNETFDAREDYSET;
+	}
+	ent = getfdnewentry();
+
+	rpg_FDS[ent]=FD;
+	rpg_IDS[ent] = -1;
+	rpg_IDSR[ent] = -1;
+
+	return LIBNETOK ;
+}
+
 /*ФȤƥåȤ򥪡ץ󤹤*/
 int rpg_serveropen(int type ,	char * service_name ){
 
@@ -747,7 +765,7 @@ int cnt;
 int rpg_send_udp(int FD ,char * command ,  char * senddata , int len){
 /*int stat;*/
 int cnt;
-int ret , n;
+int ret ;
 libnet_send_st send_st;	
 
 	if (FD == 0){
@@ -784,7 +802,7 @@ libnet_send_st send_st;
 		rpglog(RPGDEBUGLEVEL , "libnet: head fault send message errno[%n]\n" , errno);
 		if (errno == EPIPE ){
 			rpglog(RPGDEBUGLEVEL , "libnet:got socket close\n");
-			rpgclosefd(FD);
+			rpg_closefd(FD);
 		}
 		send_st.status = LIBNETSENDERR;
 		return LIBNETSENDERR;
@@ -796,7 +814,7 @@ libnet_send_st send_st;
 			rpglog(RPGDEBUGLEVEL , "libnet:fault send message errno[%n]\n" , errno);
 			if (errno == EPIPE ){
 				rpglog(RPGDEBUGLEVEL , "libnet:got socket close \n");
-				rpgclosefd(FD);
+				rpg_closefd(FD);
 			}
 			send_st.status = LIBNETHEADSENDERR;
 			return LIBNETHEADSENDERR;
@@ -810,7 +828,7 @@ libnet_send_st send_st;
 int rpg_send(int FD ,char * command ,  char * senddata , int len){
 /*int stat;*/
 int cnt;
-int ret , n ;
+int ret ;
 libnet_send_st send_st;	
 
 	if (FD == 0){
@@ -846,7 +864,7 @@ libnet_send_st send_st;
 		rpglog(RPGDEBUGLEVEL , "libnet:head faulr send errno[%n]\n" , errno);
 		if (errno == EPIPE ){
 			rpglog(RPGDEBUGLEVEL , "libnet:got socket close \n");
-			rpgclosefd(FD);
+			rpg_closefd(FD);
 		}
 		send_st.status = LIBNETSENDERR;
 		return LIBNETSENDERR;
@@ -858,7 +876,7 @@ libnet_send_st send_st;
 			rpglog(RPGDEBUGLEVEL , "libnet:fault send errno[%n]\n" , errno);
 			if (errno == EPIPE ){
 				rpglog(RPGDEBUGLEVEL , "libnet:got socket close\n");
-				rpgclosefd(FD);
+				rpg_closefd(FD);
 			}
 			send_st.status = LIBNETHEADSENDERR;
 			return LIBNETHEADSENDERR;
@@ -907,7 +925,7 @@ libnet_send_st send_st;
 
 *************************************/
 /*ǡ׵᥵ʬɤ߹*/
-int read_packet(int FD	 , int requestlen , int *read_len , char * buf , int bufsizei , int timeout){
+static int read_packet(int FD	 , int requestlen , int *read_len , char * buf , int bufsizei , int timeout){
 
 
 
@@ -960,7 +978,7 @@ int read_packet(int FD	 , int requestlen , int *read_len , char * buf , int bufs
 	} 
 	/*Хȼ*/
 	if (ret == 0){
-		rpgclosefd (FD);
+		rpg_closefd (FD);
 		return LIBNETLENGTHZERO;
 	}
 	/*ǡʬäϥ꥿󤹤*/
@@ -1036,7 +1054,7 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt++){
 			if (s_Poll[cnt].revents){
 				if (s_Poll[cnt].revents > POLLERR){
-					rpgclosefd (s_Poll[cnt].fd);
+					rpg_closefd (s_Poll[cnt].fd);
 					return LIBNETPOLLHUP;
 				}
 				*resfd = s_Poll[cnt].fd;
@@ -1050,17 +1068,18 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 						(struct sockaddr *)&caddr, &len)) < 0) {
 						perror("accept");
 						rpglog(RPGDEBUGLEVEL , "libnet:acept error WHY!![%d]\n" , errno);
-						if (rpg_FDS[new_fd] > 0)rpgclosefd(rpg_FDS[new_fd]);
+						if (rpg_FDS[new_fd] > 0)rpg_closefd(rpg_FDS[new_fd]);
 						return LIBNETCONNECTIONERROR;/*exit(1);*/
 					}
 					/*ƣĤϥǡ˳Ǽ*/
 					memcpy (command , LIBNETCONNECTREQ , COMMANDLEN);
 					memcpy (data , &rpg_FDS[new_fd] , sizeof (int));
+					memcpy (&(data[sizeof (int)]),&caddr,sizeof(caddr));
 					*resfd = rpg_FDS[new_fd];
 					*length = sizeof (int);
 					/*׵ƥץȤ饳ͥȤȤ֤*/
-					rpglog (RPGDEBUGLEVEL , "libnet:return that have  connect [%d]-[%d]\n"
-					, new_fd , rpg_FDS[new_fd]);
+					rpglog (RPGDEBUGLEVEL , "libnet:return that have  connect [%d]-[%d] \n"
+					, new_fd , rpg_FDS[new_fd] );
 					return (LIBNETCONNECTREQUESTU);
 				}
 				/*³׵ץץȤơƣĤϿ*/
@@ -1074,7 +1093,7 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 						(struct sockaddr *)&caddr, &len)) < 0) {
 						/*ԤФƼԤ֤*/
 						rpglog (RPGDEBUGLEVEL , "libnet:acept error WHY!![%d]\n" , errno);
-						if (rpg_FDS[new_fd] > 0)rpgclosefd(rpg_FDS[new_fd]);
+						if (rpg_FDS[new_fd] > 0)rpg_closefd(rpg_FDS[new_fd]);
 						return LIBNETCONNECTIONERROR;
 					}
 					/*ץ׵᤬ΣţԣңУǤƤ뤫ǧ*/
@@ -1084,25 +1103,37 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 					if (ret < 0 ){
 						/*ǡʤȤϥͥ󥨥顼Ȥƽ*/
 						rpglog (RPGDEBUGLEVEL , "libnet:it must have packet[%d]-[%d]\n" , ret , timeout);
-						rpgclosefd (rpg_FDS[new_fd]);
+						rpg_closefd (rpg_FDS[new_fd]);
 						return LIBNETCONNECTIONDATAERR;
 					}
 					/*Ƥǧ*/
 					if (memcmp (data , LIBNETASSEPT , strlen(LIBNETASSEPT))!=0){
 						rpglog (RPGDEBUGLEVEL , "libnet:test is is true\n");
-						rpgclosefd (rpg_FDS[new_fd]);
+						rpg_closefd (rpg_FDS[new_fd]);
 						return LIBNETCONNECTIONDATAERR;
 					}
 					/*³λΤ³ƣĤǡ
 					³Ȥ֤*/
 					memcpy (data , &rpg_FDS[new_fd] , sizeof (int));
+					memcpy (&(data[sizeof (int)]),&caddr,sizeof(caddr));
 					*resfd = rpg_FDS[new_fd];
 					*length = sizeof (int);
 					memcpy (command , LIBNETCONNECTREQ , COMMANDLEN);
-					rpglog (RPGDEBUGLEVEL , "libnet:return connect if it was sucsess[%d]-[%d]\n"
-					, new_fd , rpg_FDS[new_fd]);
+					rpglog (RPGDEBUGLEVEL , "libnet:return connect if it was sucsess[%d]-[%d] from[%s]\n"
+					, new_fd , rpg_FDS[new_fd],inet_ntoa(caddr.sin_addr));
 					return LIBNETCONNECTREQUESTT;
 				}
+				cnt2 = test_fd(s_Poll[cnt].fd);
+				/*rpg_IDSäңףƣļȤƽ*/
+				if (rpg_IDS[cnt2]==-1){
+					/*RAW data äϤΤޤ޼ʤǤ*/
+					*length=0;
+					*resfd=s_Poll[cnt].fd;
+					memcpy (command , LIBNETRAWDATARECEVE , COMMANDLEN);
+					rpglog (RPGDEBUGLEVEL , "libnet:Raw data receve FD%d length%d\n",
+						*resfd,*length);
+					return LIBNETRAWRESEVE;
+				}
 				/*ǡξ硢إåᤷƽ*/
 				ret = read (s_Poll[cnt].fd , &packet_head , sizeof (libnet_send_st));
 				if (ret < 0){
@@ -1113,7 +1144,7 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 				}
 				/*Хȼ*/
 				if (ret == 0){
-					rpgclosefd(s_Poll[cnt].fd);
+					rpg_closefd(s_Poll[cnt].fd);
 					return LIBNETLENGTHZERO;
 				}
 				/*Ĺإå礭ʤ*/
@@ -1149,7 +1180,6 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 					return LIBNETNOREQLEN;
 				}
 				/*μν򤹤롣*/
-				cnt2 = test_fd(s_Poll[cnt].fd);
 				if (cnt2 < 0)return LIBNETERROTHER;
 				
 				if (rpg_FDS[cnt2] <= 0){
@@ -1223,22 +1253,30 @@ char rpgPname[20];
 ФˤȤäƽʥ顼ˤĤƤϥФؤ
 ͤƤǤ礦
 
-int loginit( char * pname );
+int loginit( char * pname );		СȤ³
+int rpg_loginit( char * pname  , char * sockname);	ФȤ³å̾
 int rpg_dbglvl(int dbglvl);
-int rpglog(int DBlev , char * errbuf , ...);
-int rpg_gamelog();
+int rpglog(int DBlev , char * errbuf , ...);СäƤƤ餦
+char *  rpg_setlogfile(char * filename){ Ф³ʤȤϤδؿǥե̾Ͽ
+int rpg_gamelog();  ˽񤭽Ф 	
+int rpg_logclose(){	եΥ
 
 ФΤߤȤΤ
-int rpg_gamelog( int dbglvl ,char * msg );
+char *  rpg_setlogfile(char * filename){ ץ̾Ͽ
+int rpg_gamelog( int dbglvl ,char * msg ); ΥϤ
 줿ǥХå٥ȥ٤ƤȤƤ錄
+int rpg_receveLogwrite(char * readbuf);	ǡ񤭽Ф
 int rpg_dbglvl(int dbglvl);
+int rpg_logclose(){	եΥ
 ˤäƥ󥰥٥򥻥åȤ롣
 
 
 
 饤ȤϤޤ
 
-int loginit( char * pname );
+int loginit( char * pname );  ФȤ³
+ޤ
+int rpg_loginit( char * pname  , char * sockname);	ФȤ³å̾
 
 ˤơФ³򤹤
 饤Ȥξ³˼ԤΤǥե˽񤭹褦ˤʤ
@@ -1274,6 +1312,20 @@ int rpg_dbglvl(int dbglvl);
 ƥ٥˱ƵưѤ٤
 
 **********************************************************/
+
+static char * rpg_logsockname(char * socname){
+    static char gsockname[MAXPATHLEN];
+    static int flag=0;
+    if (flag==0){
+        bzero(gsockname,MAXPATHLEN);
+        strncpy(gsockname,LIBNETLOGSOCNAME,MAXPATHLEN);
+        flag=1;
+    }
+    if (socname!=NULL) strncpy(gsockname,socname,MAXPATHLEN);
+    return gsockname;
+}
+
+/*ե̾ꡡե̾*/
 char *  rpg_setlogfile(char * filename){
 	static int fl=0;
 	static char gfln[MAXPATHLEN];
@@ -1287,6 +1339,21 @@ char *  rpg_setlogfile(char * filename){
 	return gfln;
 }
 
+/* С
+ * 饤ȤΡRPGLOGREQ򤦤
+ * ΥǡȤϤ
+ * ǥХå٥򥻥åȤƥ󥰤*/
+int rpg_receveLogwrite(char * readbuf){
+	int dbglev;
+
+	dbglev=ntohl(*((int *)readbuf));
+	return rpg_gamelog(dbglev, &(readbuf[sizeof(int)]));
+}
+
+/* 񤭽ФΥǥХå
+ * rpg_setlogfile()ˤƥåȤե̾Ф
+ * УɣĤղäƥϤ롣
+ * ߤϤΥåڤӥơȵǽ̵*/
 int rpg_gamelog(int dbglvl ,char * tmpbuf ){
 	
 static FILE *FD =NULL;
@@ -1310,11 +1377,12 @@ char filename[LOGBUFFSIZE];
 		if (FD == NULL){
 			/*ǥХåե뤬Ƥʤг*/
 			sprintf (filename ,	 "%s.%d" , rpg_setlogfile(NULL) , getpid());
-			FD = fopen (filename , "w");
+			FD = fopen (filename , "w+");
 			if (FD == NULL ){
 				/*եΥץ*/
 				write(2,"cant open errlog file\n" , strlen("cant open errlog file\n"));
 				perror ("open ");	
+				write(2,filename , strlen(filename));
 				write(2,errerbufffer , strlen(errerbufffer));
 				write(2,tmpbuf , strlen (tmpbuf));	/*ʤΤ
 							ɸϤ˽Ф*/
@@ -1351,6 +1419,7 @@ char filename[LOGBUFFSIZE];
 	return RPG_OK ;
 }
 
+/*եΥ*/
 int rpg_logclose(){
 	rpg_gamelog(-1,"");
 	return RPG_OK;
@@ -1361,6 +1430,8 @@ int rpg_logclose(){
 int rpg_loginit( char * pname  , char * sockname){
 	int ret = 0;
 	if (sockname != ""){
+		rpg_logsockname(sockname);
+
 		if ( logconnect == 0){/*⤷Ф³Ƥʤä³򤹤*/
 			if ( rpglogfd == 0){
 				if ( (ret = rpg_usoc_client_open(sockname)) < 0){
@@ -1387,10 +1458,10 @@ int rpg_loginit( char * pname  , char * sockname){
 /*顼ν򤹤*/
 int loginit( char * pname ){
 	int ret = 0;
-	if (LIBNETLOGSOCNAME!=""){
+	if (rpg_logsockname(NULL)!=""){
 		if ( logconnect == 0){/*⤷Ф³Ƥʤä³򤹤*/
 			if ( rpglogfd == 0){
-				if ( (ret = rpg_usoc_client_open(LIBNETLOGSOCNAME)) < 0){
+				if ( (ret = rpg_usoc_client_open(rpg_logsockname(NULL))) < 0){
 					rpglog (RPGDEBUGLEVEL , "libnet:cant connect log server\n");
 					return ret;
 				}
@@ -1440,7 +1511,7 @@ va_list ap;
 	}else{
 		/*֤򥻥åȤ*/
 		timebuf = localtime(&timeb);
-		sprintf (errerbufffer , "%d/%d %d:%d:%d " , timebuf->tm_mon +1 ,
+		snprintf (errerbufffer ,LOGBUFFSIZE, "%d/%d %d:%d:%d " , timebuf->tm_mon +1 ,
 		timebuf->tm_mday , timebuf->tm_hour , timebuf->tm_min , timebuf->tm_sec );
 	}
 	/*դ᤹*/
@@ -1453,12 +1524,15 @@ va_list ap;
 			if (rpg_dbglvl(0)%2)printf (errerbufffer);
 			if (rpg_dbglvl(0)%2)printf (tmpbuf);
 		}
+		rpg_gamelog(DBlev , tmpbuf);
 		return LIBNETNOTOPENLOG;
 	}
 	/*ƣĤϿƤ뤫ǧ*/
 	if (test_fd(rpglogfd) < 0){
 		if ((ret = loginit (rpgPname)) < 0){
 			logconnect = 0 ;
+			rpg_gamelog(DBlev , tmpbuf);
+			loginit( rpgPname);
 			return (ret);
 		}
 	}
@@ -1476,7 +1550,16 @@ va_list ap;
 			strncat (&filename[sizeof (int)] , tmpbuf ,LOGBUFFSIZE-strlen(rpgPname)-1);
 		}
 		ret = rpg_send(rpglogfd , RPGLOGREQ , filename	, sizeof(int)+strlen (&filename[sizeof (int)]));
+		if (ret <0){
+			rpg_closefd(rpglogfd);
+			rpglogfd=0;
+			logconnect=0;
+			loginit( rpgPname);
+		}
 	}else{
+		rpg_gamelog(DBlev , tmpbuf);
+		logconnect=0;
+		loginit( rpgPname);
 		return LIBNETNOFD;
 	}
 	/*ǥХå٥뤬λɸ२顼ˤФ*/
@@ -1580,6 +1663,7 @@ char *	RPG_shmctl_init(shment * shmentry , int entrynom  , int *ctlshmid ){
 		}
 	}
 	/*ͭΥꥢ*/
+	/*ޤ衪*/
 	memset (RPG_shm , (int)NULL , LIBNETSHMIDMAX * sizeof(RPG_shmunit));
 	RPG_shm[0].shm_id = *ctlshmid;
 	RPG_shm[0].size = LIBNETSHMIDMAX * sizeof(RPG_shmunit);
@@ -1821,7 +1905,7 @@ int ret,cnt ;
 	rpglog (RPGDEBUGLEVEL , "libnet:rpg_mclose:cant find map to close\n");
 }
 int rpg_mlength(char * filepath){
-int ret,cnt ;
+int cnt ;
 	/*Ͽ򸡺Ƥ*/
     for (cnt = 0 ; cnt < rpgmmatnom;cnt++){
         if (strncmp ( filepath , libnet_mmap[cnt].fpath , MAXPATHLEN)==0){
@@ -1986,8 +2070,8 @@ errtable errmessage[MAXERRNOM] = {
 	"The memory was aredy open on same ID \n Dont warry you dont have to do any thing.i\nthis is worninng." } 	,
 	/*Ǥ˶꤬ͭץ󤵤Ƥ*/
 
-{  10009,"Got connect request",""},		/*³׵*/
-{  10008,"Got connect request",""},		/*³׵*/
+{  10009,"Got connection request",""},		/*³׵*/
+{  10008,"Got connection request",""},		/*³׵*/
 {  10007,"Open FD",""},		/*ƣĤ򥪡ץ󤷤*/
 {  10006,"FD was not open",""},		/*ץ󤵤Ƥʤ*/
 {  10005,"Got signal befor wait event",""},		/*׵ᤵ줿٥ȤΤɤ줫˥ʥ뤬ȯ*/
@@ -2136,7 +2220,9 @@ int cnt;
 	for (cnt = 0 ; errmessage[cnt].rpg_errno != 0 ;cnt++){
 		if (errmessage[cnt].rpg_errno == l_errno){
 			ret=errmessage[cnt].title;
-			message=errmessage[cnt].message;
+			if (message !=NULL){
+				*message=errmessage[cnt].message;
+			}
 			break;
 		}
 	}
