@@ -93,10 +93,11 @@
 #define TRUE 1
 #endif
 
-#define LIBNET
 #define LIBNET
 #define LIBNET
 #define LIBNETOREDYOPEN			20001		/*Ǥ˶꤬ͭץ󤵤Ƥ*/	
+#define LIBNETFDAREDYSET		20002		/*ǤˣƣĤоݤȤϿƤ*/
+#define LIBNETRAWRESEVE			10010		/*RAWFDȯ*/
 #define LIBNETCONNECTREQUESTT	10009		/*³׵*/
 #define LIBNETCONNECTREQUESTU	10008		/*³׵*/
 #define LIBNETFDOPEN			10007		/*ƣĤ򥪡ץ󤷤*/
@@ -177,6 +178,7 @@
 #define LIBNETASSEPT			"LIBNETASSEPT"	 
 #define LIBNETCONNECTREQ		"CONNECTREQ"	/*³׵*/
 #define POLLTIMEUP          	"POLLTIMEUP"            /**/
+#define LIBNETRAWDATARECEVE			"RAWDATAREC"
 #define LIBNETWTIMEUP2			"LIBNETIMUP"
 /*		ʸ󤬤뤳,
 		length
@@ -270,7 +272,7 @@ typedef struct{
 				LIBNETNOFDȥ꡼¸ߤʤ
 */
 
-			int test_fd(int fd );
+static	int test_fd(int fd );
 
 
 /*************************************************************************/
@@ -290,7 +292,7 @@ typedef struct{
 		Ƥƣĥǡ١ȥ
 */
 
-int getfdnewentry(void);
+static int getfdnewentry(void);
 
 
 
@@ -307,7 +309,7 @@ int getfdnewentry(void);
 		ʤ
 */
 
-void	rpgclosefd(int FD);
+void	rpg_closefd(int FD);
 
 
 /**************************************************************************/
@@ -341,7 +343,7 @@ void rpg_fd_allclose(void);
 
 */
 
-int clear_fddatabase(void);
+static int clear_fddatabase(void);
 
 
 
@@ -511,7 +513,7 @@ int rpg_eventloop( char * data , int * leng	 , struct  timeval timeout );
 
 	֤͡
 */
-int read_packet (int FD	 , int requestlen , int *read_len , char * buf , int bufsizei , int timeout);
+static int read_packet (int FD	 , int requestlen , int *read_len , char * buf , int bufsizei , int timeout);
 
 
 
